@@ -38,8 +38,16 @@
 #include "kscopeconfig.h"
 #include "graphprefdlg.h"
 
+/** The currently supported version of saved call-tree files. */
 #define FILE_VERSION		5
 
+/** Window flags for call-tree widgets. */
+#define CALL_TREE_W_FLAGS \
+	WStyle_Customize | \
+	WStyle_NormalBorder | \
+	WStyle_Title | \
+	WDestructiveClose
+
 /** File Name index for the file name generation */
 int CallTreeDlg::s_nFileNameIndex = 0;
 
@@ -49,11 +57,8 @@ int CallTreeDlg::s_nFileNameIndex = 0;
  * @param	szName	The widget's name
  */
 CallTreeDlg::CallTreeDlg(QWidget* pParent, const char* szName) :
-	CallTreeLayout(pParent, szName)
+	CallTreeLayout(pParent, szName, CALL_TREE_W_FLAGS)
 {
-	// Make sure the dialog object is deleted when the dialog is closed
-	setWFlags(getWFlags() | WDestructiveClose);
-
 	// Set button pixmaps
 	m_pCalledButton->setPixmap(GET_PIXMAP(CalledTree));
 	m_pCallingButton->setPixmap(GET_PIXMAP(CallingTree));
@@ -130,7 +135,7 @@ void CallTreeDlg::closeEvent(QCloseEvent* pEvent)
 		QFile::remove(m_sFilePath);
 		
 	emit closed(this);
-	QDialog::closeEvent(pEvent);
+	QWidget::closeEvent(pEvent);
 }
 
 extern void yyinit(CallTreeDlg*, FILE*, Encoder*);
