@@ -58,6 +58,8 @@ public:
 	void gotoLine(uint);
 	void clear();
 	
+	virtual bool getTip(QListViewItem*, QString&);
+	
 public slots:
 	void slotDataReady(FrontendToken*);
 	void slotCtagsFinished(uint);
@@ -78,7 +80,7 @@ signals:
 protected:
 	virtual void resizeEvent(QResizeEvent*);
 	virtual void processItemSelected(QListViewItem*);
-
+	
 private:
 	/** An array of pointers to the tag list items, sorted by the line
 		number. */
@@ -98,9 +100,6 @@ private:
 	
 	/** Stores the requested line number during Ctags operation. */
 	uint m_nPendLine;
-	
-	/** A tool-tip for the list entries. */
-	CtagsToolTip* m_pToolTip; 
 
 private slots:
 	void slotSortChanged(int);
