@@ -57,6 +57,7 @@ EditorPage::EditorPage(KTextEditor::Document* pDoc, QPopupMenu* pMenu,
 {
 	KTextEditor::PopupMenuInterface* pMenuIf;
 	KTextEditor::ViewCursorInterface* pCursorIf;
+	KTextEditor::MarkInterfaceExtension* pMarkIf;
 	
 	// Create code-completion objects (will be deleted by QObject destructor)
 	m_pCompletion = new SymbolCompletion(this, this);
@@ -105,6 +106,15 @@ EditorPage::EditorPage(KTextEditor::Document* pDoc, QPopupMenu* pMenu,
 		connect(m_pView, SIGNAL(cursorPositionChanged()), this,
 			SLOT(slotCursorPosChange()));
 	}
+	
+	// Monitor bookmarks
+	pMarkIf = dynamic_cast<KTextEditor::MarkInterfaceExtension*>(m_pDoc);
+	if (pMarkIf) {
+		connect(m_pDoc, SIGNAL(markChanged(KTextEditor::Mark, 
+			KTextEditor::MarkInterfaceExtension::MarkChangeAction)), this,
+			SLOT(slotMarkChanged(KTextEditor::Mark,
+			KTextEditor::MarkInterfaceExtension::MarkChangeAction)));
+	}
 }
 
 /**
@@ -209,18 +219,25 @@ void EditorPage::save()
 /**
  * Closes an edited file.
  * @param	bForce	true to close the file regardless of any modifications, 
- *			false to prompt the user in case of unsaved chnages 
+ *					false to prompt the user in case of unsaved chnages 
  * @return	true if the file has been closed, false if the user has aborted
  */
 bool EditorPage::close(bool bForce)
 {
+	QString sPath;
+	
 	// To override the prompt-on-close behaviour, we need to mark the file
 	// as unmodified
 	if (bForce)
 		m_pDoc->setModified(false);
 	
 	// Close the file, unless the user aborts the action
-	return m_pDoc->closeURL();
+	sPath = m_pDoc->url().path();
+	if (!m_pDoc->closeURL())
+		return false;
+		
+	emit fileClosed(sPath);
+	return true;
 }
 
 /**
@@ -247,6 +264,28 @@ void EditorPage::setEditorFocus()
 	slotCursorPosChange();
 }
 
+/**
+ * Sets the keyboard focus to the tag list.
+ * This method is called when the "Go To Tag" menu command is invoked.
+ */
+void EditorPage::setTagListFocus()
+{
+	m_pCtagsList->slotSetFocus();
+}
+
+/**
+ * Sets a bookmark at the given line.
+ * @param	nLine	The line to mark
+ */
+void EditorPage::addBookmark(uint nLine)
+{
+	KTextEditor::MarkInterface* pMarkIf;
+	
+	pMarkIf = dynamic_cast<KTextEditor::MarkInterface*>(m_pDoc);
+	if (pMarkIf)
+		pMarkIf->setMark(nLine, KTextEditor::MarkInterface::markType01);
+}
+
 /**
  * Returns the currently selected text in an open file.
  * @return	The selected text, or a null string if no text is currently
@@ -544,6 +583,20 @@ bool EditorPage::setCursorPos(uint nLine, uint nCol)
 	return true;
 }
 
+void EditorPage::setTabWidth(uint nTabWidth)
+{
+	Kate::Document* pKateDoc;
+	Kate::Command* pKateCmd;
+	QString sCmd, sResult;
+	
+	pKateDoc = dynamic_cast<Kate::Document*>(m_pDoc);
+	if ((pKateDoc) &&
+		(pKateCmd = pKateDoc->queryCommand("set-tab-width"))) {
+		sCmd.sprintf("set-tab-width %u", nTabWidth);
+		pKateCmd->exec((Kate::View*)m_pView, sCmd, sResult);
+	}
+}
+
 /**
  * Called when a document has completed loading.
  * Determines the file's properties and refreshes the tag list of the editor
@@ -652,4 +705,24 @@ void EditorPage::slotCursorPosChange()
 	m_pCompletion->abort();
 }
 
+/**
+ * Called when a mark is added or removed.
+ * If the changed mark is a bookmark, the bookmarkChanged() signal is emitted.
+ * This slot is connected to the markChanged() signal of the document object.
+ * @param	mark	The changed mark
+ * @param	act		Whether the mark was added or removed
+ */
+void EditorPage::slotMarkChanged(KTextEditor::Mark mark,
+	KTextEditor::MarkInterfaceExtension::MarkChangeAction act)
+{
+	uint nLine;
+	
+	if (mark.type == KTextEditor::MarkInterface::markType01) {
+		nLine = mark.line + 1; // Convert to 1-based line numbers
+		emit bookmarkChanged(m_pDoc->url().path(), nLine,
+			getLineContents(nLine),
+			act == KTextEditor::MarkInterfaceExtension::MarkAdded);
+	}
+}
+
 #include "editorpage.moc"
