@@ -313,7 +313,7 @@ void EditorTabs::applyPrefs()
  * open.
  * @param	list	The list to fill
  */
-void EditorTabs::getOpenFiles(OpenFileList& list)
+void EditorTabs::getOpenFiles(FileLocationList& list)
 {
 	int i;
 	EditorPage* pPage;
@@ -333,6 +333,23 @@ void EditorTabs::getOpenFiles(OpenFileList& list)
 	}
 }
 
+/**
+ */
+void EditorTabs::setBookmarks(FileLocationList& list)
+{
+	FileLocation* pLoc;
+	EditorMap::iterator itr;
+	EditorPage* pPage;
+	
+	for (pLoc = list.first(); pLoc; pLoc = list.next()) {
+		itr = m_mapEdit.find(pLoc->m_sPath);
+		if (itr != m_mapEdit.end()) {
+			pPage = *itr;
+			pPage->addBookmark(pLoc->m_nLine);
+		}
+	}
+}
+
 /**
  * Removes an editor page.
  * If there are unsaved changes, the user is prompted, and the file is closed 
