@@ -58,7 +58,8 @@ public:
 	void removeCurrentPage();
 	bool removeAllPages();
 	void applyPrefs();
-	void getOpenFiles(OpenFileList&);
+	void getOpenFiles(FileLocationList&);
+	void setBookmarks(FileLocationList&);
 	
 public slots:
 	void slotRemovePage(QWidget*);
