@@ -27,7 +27,7 @@
 
 #include <qfileinfo.h>
 #include <qdir.h>
-#include <kmessagebox.h>
+#include <klocale.h>
 #include "frontend.h"
 
 /**
@@ -81,8 +81,8 @@ bool Frontend::run(const QString& sName, const QStringList& slArgs,
 {
 	// Cannot start if another controlled process is currently running
 	if (isRunning()) {
-		KMessageBox::error(0, sName + ": Cannot restart while another process "
-			"is still running");
+		m_sError = i18n("Cannot restart while another process is still "
+			"running");
 		return false;
 	}
 
@@ -101,10 +101,11 @@ bool Frontend::run(const QString& sName, const QStringList& slArgs,
 	// Execute the child process
 	if (!start(bBlock ? KProcess::Block : KProcess::NotifyOnExit,
 		KProcess::All)) {
-		KMessageBox::error(0, sName + ": Failed to start process");
+		m_sError = sName + i18n(": Failed to start process");
 		return false;
 	}
 	
+	m_sError = i18n("No error");
 	return true;
 }
 
@@ -121,6 +122,17 @@ void Frontend::kill()
 	emit aborted();
 }
 
+/**
+ * Lets stderr be parsed through the stdout parser.
+ */
+void Frontend::joinOutputStreams()
+{
+	disconnect(this, SIGNAL(receivedStderr(KProcess*, char*, int)), this,
+		SLOT(slotReadStderr(KProcess*, char*, int)));
+	connect(this, SIGNAL(receivedStderr(KProcess*, char*, int)), this,
+		SLOT(slotReadStdout(KProcess*, char*, int)));
+}
+
 /**
  * Appends a token to the end of the token list.
  * @param	pToken	The token to add
