@@ -85,6 +85,11 @@ public:
 		const QString& sWorkDir = "", bool bBlock = false);
 	void kill();
 		
+	/**
+	 * @return	An string describing the error which made run() fail
+	 */
+	const QString& getRunError() { return m_sError; }
+	
 signals:
 	/**
 	 * Indicates tokens can be read.
@@ -148,6 +153,9 @@ protected:
 	/** The current delimiters used for parsing the output. */
 	ParserDelim m_delim;
 	
+	/** An error string produced if run() fails. */
+	QString m_sError;
+	
 	/**
 	 * Handles a text token received on the Standard Output stream of the
 	 * controlled process.
@@ -170,6 +178,8 @@ protected:
 	 */
 	virtual void finalize() {}
 
+	void joinOutputStreams();
+
 protected slots:
 	virtual void slotProcessExit(KProcess*);
 	
