@@ -458,7 +458,8 @@ QString GraphWidget::getTip(const QPoint& ptPos, QRect& rc)
 
 /**
  * Resizes the canvas.
- * @param	size	The new size to use
+ * @param	nWidth	The new width
+ * @param	nHiehgt	The new height
  */
 void GraphWidget::resize(int nWidth, int nHeight)
 {
@@ -687,19 +688,31 @@ void GraphWidget::write(QTextStream& str, const QString& sType,
  * Removes all edges attached to a function node at the given direction.
  * Any strongly connected components that are no longer connected to that
  * function are deleted.
- * @param	pGraphNode	The node for which to remove the edges
- * @param	bOut		true for outgoing edges, false for incoming
+ * @param	pNode	The node for which to remove the edges
+ * @param	bOut	true for outgoing edges, false for incoming
  */
 void GraphWidget::removeEdges(GraphNode* pNode, bool bOut)
 {
-	QDictIterator<GraphNode> itr(m_dictNodes);
-	
 	// Remove the edges
 	if (bOut)
 		pNode->removeOutEdges();
 	else
 		pNode->removeInEdges();
 		
+	// Remove all graph components no longer connected to this one
+	removeDisconnected(pNode);
+}
+
+/**
+ * Removes all edges and nodes that are not weakly connected to the given node.
+ * This function is called to clean up the graph after edges were removed from
+ * the given node.
+ * @param	pNode	The node to which all other nodes have to be connected
+ */
+void GraphWidget::removeDisconnected(GraphNode* pNode)
+{
+	QDictIterator<GraphNode> itr(m_dictNodes);
+	
 	// Find all weakly connected components attached to this node
 	pNode->dfs();
 	
@@ -748,6 +761,11 @@ void GraphWidget::showEdgeMenu(GraphEdge* pEdge, const QPoint& ptPos)
 	m_pEdgePopup->popup(ptPos);
 }
 
+/**
+ * Redraws the widget when new instructions are available.
+ * This slot is connected to the finished() signal emitted by the dot front-
+ * end.
+ */
 void GraphWidget::slotRepaint()
 {
 	setUpdatesEnabled(true);
@@ -870,7 +888,7 @@ void GraphWidget::slotListCalled()
 	if (pNode == NULL)
 		return;
 	
-	QueryViewDlg dlg;
+	QueryViewDlg dlg(0, (QWidget*)parent());
 	
 	// Show the query view dialogue
 	dlg.query(CscopeFrontend::Called, pNode->getFunc());
@@ -878,7 +896,7 @@ void GraphWidget::slotListCalled()
 		return;
 		
 	// The OK button was clicked, replace current calls with the listed ones
-	removeEdges(pNode, true);
+	pNode->removeOutEdges();
 	
 	QueryView::Iterator itr;
 	CallData data;
@@ -895,7 +913,8 @@ void GraphWidget::slotListCalled()
 		addCall(data);
 	}
 	
-	// Redraw the graph
+	// Clean-up and redraw the graph
+	removeDisconnected(pNode);
 	draw();
 }
 
@@ -955,7 +974,7 @@ void GraphWidget::slotListCalling()
 	if (pNode == NULL)
 		return;
 	
-	QueryViewDlg dlg;
+	QueryViewDlg dlg(0, (QWidget*)parent());
 	
 	// Show the query view dialogue
 	dlg.query(CscopeFrontend::Calling, pNode->getFunc());
@@ -963,7 +982,7 @@ void GraphWidget::slotListCalling()
 		return;
 		
 	// The OK button was clicked, replace current calls with the listed ones
-	removeEdges(pNode, false);
+	pNode->removeInEdges();
 	
 	QueryView::Iterator itr;
 	CallData data;
@@ -980,7 +999,8 @@ void GraphWidget::slotListCalling()
 		addCall(data);
 	}
 	
-	// Redraw the graph
+	// Clean-up and redraw the graph
+	removeDisconnected(pNode);
 	draw();
 }
 
@@ -1042,8 +1062,11 @@ void GraphWidget::slotRemoveNode()
 	pNode = dynamic_cast<GraphNode*>(m_pMenuItem);
 	if (pNode == NULL)
 		return;
-		
-	// Remove the node (also deletes the object)
+	
+	// Remove all incoming edges
+	pNode->removeInEdges();
+	
+	// Remove the node (also deletes the object and its outgoing edges)
 	m_dictNodes.remove(pNode->getFunc());
 	
 	// Redraw the graph
@@ -1070,7 +1093,7 @@ void GraphWidget::slotMultiCallDetails()
 	nType = IS_TRUE(GET_NODE_ATTR(pNode->getNode(), "kscope_multi_called")) ?
 		CscopeFrontend::Called : CscopeFrontend::Calling;
 		
-	QueryViewDlg dlg;
+	QueryViewDlg dlg(0, (QWidget*)parent());
 	
 	dlg.query(nType, sFunc);
 	dlg.exec();*/
