@@ -62,6 +62,7 @@
 #include "symbolcompletion.h"
 #include "queryviewdlg.h"
 #include "graphwidget.h"
+#include "makedlg.h"
 #include "welcomedlg.h"
 
 /**
@@ -75,7 +76,10 @@ KScope::KScope(QWidget* pParent, const char* szName) :
 	m_sCurFilePath(""),
 	m_nCurLine(0),
 	m_pProgressDlg(NULL),
-	m_bUpdateGUI(true)
+	m_bUpdateGUI(true),
+	m_bCscopeVerified(false),
+	m_bRebuildDB(false),
+	m_pMakeDlg(NULL)
 {
 	QString sPath;
 
@@ -189,6 +193,9 @@ KScope::~KScope()
 	delete m_pEditMgr;
 	delete m_pCscopeBuild;
 	delete m_pProjMgr;
+	
+	if (m_pMakeDlg != NULL)
+		delete m_pMakeDlg;
 }
 
 /**
@@ -216,6 +223,8 @@ void KScope::initActions()
 	m_pExtEditAction = new KAction(i18n("Edit in E&xternal Editor"),
 		KShortcut("Ctrl+E"), this,
 		SLOT(slotExtEdit()), actionCollection(), "edit_external_editor");
+	(void)new KAction(i18n("Go To Tag"), KShortcut("Ctrl+Shift+T"), this,
+		SLOT(slotGotoTag()), actionCollection(), "edit_goto_tag");
 	(void)new KAction(i18n("Complete Symbol"), KShortcut("Ctrl+Space"), this,
 		SLOT(slotCompleteSymbol()), actionCollection(), "edit_comp_symbol");
 		
@@ -224,15 +233,19 @@ void KScope::initActions()
 		SLOT(slotCreateProject()), actionCollection(), "project_new");
 	(void)new KAction(i18n("&Open Project..."), "project_open", 0, this,
 		SLOT(slotOpenProject()), actionCollection(), "project_open");
+	(void)new KAction(i18n("Open &Cscope.out..."), 0, this,
+		SLOT(slotProjectCscopeOut()), actionCollection(),
+		"project_cscope_out");
 	(void)new KAction(i18n("Add/Remove &Files..."), 0, this,
 		SLOT(slotProjectFiles()), actionCollection(),
 		"project_add_rem_files");
 	(void)new KAction(i18n("&Properties..."), 0, this,
 		SLOT(slotProjectProps()), actionCollection(),
 		"project_properties");
-	(void)new KAction(i18n("Open &Cscope.out..."), 0, this,
-		SLOT(slotProjectCscopeOut()), actionCollection(),
-		"project_cscope_out");
+	(void)new KAction(i18n("&Make Project"), KShortcut("Ctrl+M"), this,
+		SLOT(slotProjectMake()), actionCollection(), "project_make");
+	(void)new KAction(i18n("&Remake Project"), KShortcut("Ctrl+Shift+M"), this,
+		SLOT(slotProjectRemake()), actionCollection(), "project_remake");
 	(void)new KAction(i18n("&Close Project"), 0, this,
 		SLOT(slotCloseProject()), actionCollection(), "project_close");
 
@@ -264,22 +277,25 @@ void KScope::initActions()
 
 	// Go menu
 	(void)new KAction(i18n("P&revious Result"), "up", 
-		KShortcut(QKeySequence(ALT + Key_Up)), m_pQueryWidget,
+		QKeySequence(ALT + Key_Up), m_pQueryWidget,
 		SLOT(slotPrevResult()), actionCollection(), "go_prev_result");
 	(void)new KAction(i18n("N&ext Result"), "down", 
-		KShortcut(QKeySequence(ALT + Key_Down)), m_pQueryWidget,
+		QKeySequence(ALT + Key_Down), m_pQueryWidget,
 		SLOT(slotNextResult()), actionCollection(),
 		"go_next_result");
 	(void)new KAction(i18n("&Previous Position"), "previous", 
-		KShortcut(QKeySequence(ALT + Key_Left)), m_pQueryWidget,
+		QKeySequence(ALT + Key_Left), m_pQueryWidget,
 		SLOT(slotHistoryPrev()), actionCollection(), "go_prev_pos");
 	(void)new KAction(i18n("&Next Position"), "next", 
-		KShortcut(QKeySequence(ALT + Key_Right)), m_pQueryWidget,
+		QKeySequence(ALT + Key_Right), m_pQueryWidget,
 		SLOT(slotHistoryNext()), actionCollection(),
 		"go_next_pos");
 	(void)new KAction(i18n("Position &History"), 0, KShortcut("Ctrl+h"),
 		this, SLOT(slotHistoryShow()), actionCollection(),
 		"go_history");
+	(void)new KAction(i18n("&Global Bookmarks"), 0, 
+		QKeySequence(CTRL + SHIFT + Key_G), m_pQueryWidget,
+		SLOT(slotBrowseBookmarks()), actionCollection(), "go_bookmarks");
 
 	// View menu
 	m_pToggleFileViewAction = new KToggleAction(i18n("Show/Hide File List"),
@@ -294,16 +310,16 @@ void KScope::initActions()
 		"view_detailed", KShortcut("Ctrl+'"), m_pEditTabs,
 		SLOT(slotToggleTagList()), actionCollection(),
 		"view_toggle_tag_list");
-	
+			
 	// Window menu
 	(void)new KAction(i18n("Close &All"), 0, 0, this,
 		SLOT(slotCloseAllWindows()), actionCollection(), "window_close_all");
 	(void)new KAction(i18n("Go &Left"), 0, 
-		KShortcut(QKeySequence(ALT + SHIFT + Key_Left)),
+		QKeySequence(ALT + SHIFT + Key_Left),
 		m_pEditTabs, SLOT(slotGoLeft()), actionCollection(),
 		"window_go_left");
 	(void)new KAction(i18n("Go &Right"), 0,
-		KShortcut(QKeySequence(ALT + SHIFT + Key_Right)),
+		QKeySequence(ALT + SHIFT + Key_Right),
 		m_pEditTabs, SLOT(slotGoRight()), actionCollection(),
 		"window_go_right");
 		
@@ -619,7 +635,7 @@ void KScope::slotQueryFile()
 /**
  * Handles the "Cscope->Find Including Files..." menu command.
  * Prompts the user for a file name, and initiates a query to find all files
- * having an #include directive to that file.
+ * having an '#include' directive to that file.
  */
 void KScope::slotQueryIncluding()
 {
@@ -671,7 +687,7 @@ void KScope::slotRebuildDB()
 {
 	if (!m_pProjMgr->dbExists()) {
 		m_pProgressDlg = new ProgressDlg(i18n("KScope"), i18n("Please wait "
-			"while the database is built for the first time"), this);
+			"while KScope builds the database"), this);
 		m_pProgressDlg->setAllowCancel(false);
 		m_pProgressDlg->setValue(0);
 	}
@@ -703,7 +719,10 @@ void KScope::slotConfigure()
 	connect(&dlg, SIGNAL(applyPref()), this, SLOT(slotApplyPref()));
 
 	// Show the dialog
-	dlg.exec();
+	if (dlg.exec() == QDialog::Accepted) {
+		// Verify Cscope's installation
+		verifyCscope();
+	}
 }
 
 /**
@@ -828,20 +847,50 @@ void KScope::openProject(const QString& sDir)
 	m_pProjMgr->fillList(m_pFileList);
 	m_pFileList->setUpdatesEnabled(true);
 	
-	// Reload previously-open files
+	// Reload previously-open files and their bookmarks
 	loadOpenFiles();
-		
+	loadBookmarks();
+	
 	// Load previously stored queries
 	m_pProjMgr->getQueryFiles(slQueryFiles);
-	m_pQueryWidget->loadLockedQueries(m_pProjMgr->getPath(), slQueryFiles);
+	m_pQueryWidget->loadPages(m_pProjMgr->getPath(), slQueryFiles);
 	
 	// Load previously stored call trees
 	m_pProjMgr->getCallTreeFiles(slCallTreeFiles);
 	m_pCallTreeMgr->loadOpenDialogs(m_pProjMgr->getPath(), slCallTreeFiles);
 	
 	// Rebuild the cross-reference database
-	if (isAutoRebuildEnabled())
+	if (isAutoRebuildEnabled()) {
+		// If Cscope installation was not yet verified, postpone the build
+		// process
+		if (m_bCscopeVerified)
 		slotRebuildDB();
+		else
+			m_bRebuildDB = true;
+	}
+}
+
+/**
+ * Opens the most recently used project.
+ * This method is called when KScope starts, but only if the relevant 
+ * configuration flag (Reload Last Project) is set.
+ */
+void KScope::openLastProject()
+{
+	const QStringList slProjects = Config().getRecentProjects();
+	QString sPath;
+	
+	if (slProjects.empty())
+		return;
+		
+	sPath = *slProjects.begin();
+
+	// Projects already loaded on other instances of KScope should not be
+	// reloaded.
+	if (ProjectManager::isGloballyOpen(sPath))
+		return;
+		
+	openProject(sPath);
 }
 
 /**
@@ -851,7 +900,7 @@ void KScope::openProject(const QString& sDir)
  */
 void KScope::loadOpenFiles()
 {
-	OpenFileList list;
+	FileLocationList list;
 	FileLocation* pLoc;
 	EditorPage* pPage;
 	
@@ -882,6 +931,18 @@ void KScope::loadOpenFiles()
 	}
 }
 
+/**
+ * Restores bookmarks saved as part of the project's session.
+ * This function is called whenever a project is loaded.
+ */
+void KScope::loadBookmarks()
+{
+	FileLocationList list;
+	
+	m_pProjMgr->getBookmarks(list);
+	m_pEditTabs->setBookmarks(list);
+}
+
 /**
  * Shows or hides the query dock window.
  * This function is only called internally, not as a result of a user's
@@ -976,6 +1037,23 @@ void KScope::parseCmdLine(KCmdLineArgs* pArgs)
 	}
 }
 
+/**
+ * Starts a shell script to ensure that Cscope is properly installed and to
+ * extract the supported command-line arguments. 
+ */
+void KScope::verifyCscope()
+{
+	CscopeVerifier* pVer;
+	
+	statusBar()->message(i18n("Verifying Cscope installation..."));
+	
+	pVer = new CscopeVerifier();
+	connect(pVer, SIGNAL(done(bool, uint)), this,
+		SLOT(slotCscopeVerified(bool, uint)));
+	
+	pVer->verify();
+}
+
 /**
  * Initialises the CscopeFrontend class with the current project arguments,
  * and creates an object used for rebuilding the symbol database.
@@ -987,7 +1065,7 @@ void KScope::initCscope()
 		delete m_pCscopeBuild;
 
 	// Initialise CscopeFrontend
-	CscopeFrontend::init(m_pProjMgr->getPath(), m_pProjMgr->getArgList());
+	CscopeFrontend::init(m_pProjMgr->getPath(), m_pProjMgr->getArgs());
 
 	// Create a persistent Cscope process
 	m_pCscopeBuild = new CscopeFrontend();
@@ -995,6 +1073,8 @@ void KScope::initCscope()
 	// Show build progress information in the main status bar
 	connect(m_pCscopeBuild, SIGNAL(progress(int, int)), this,
 		SLOT(slotBuildProgress(int, int)));
+	connect(m_pCscopeBuild, SIGNAL(buildInvIndex()), this,
+		SLOT(slotBuildInvIndex()));
 	connect(m_pCscopeBuild, SIGNAL(finished(uint)), this,
 		SLOT(slotBuildFinished(uint)));
 	connect(m_pCscopeBuild, SIGNAL(aborted()), this,
@@ -1014,7 +1094,8 @@ void KScope::initCscope()
 bool KScope::slotCloseProject()
 {
 	QString sLastOpenFile;
-	OpenFileList lstOpenFiles;
+	FileLocationList lstOpenFiles;
+	FileLocationList lstBookmarks;
 	QStringList slQueryFiles;
 	QStringList slCallTreeFiles;
 	
@@ -1026,12 +1107,13 @@ bool KScope::slotCloseProject()
 	if (m_pEditTabs->count() > 0) {
 		sLastOpenFile = m_pEditTabs->getCurrentPage()->getFilePath();
 		m_pEditTabs->getOpenFiles(lstOpenFiles);
+		m_pQueryWidget->getBookmarks(lstBookmarks);
 		if (!m_pEditTabs->removeAllPages())
 			return false;
 	}
 	
 	// Store locked queries, and close all query pages
-	m_pQueryWidget->saveLockedQueries(m_pProjMgr->getPath(), slQueryFiles);
+	m_pQueryWidget->savePages(m_pProjMgr->getPath(), slQueryFiles);
 	m_pQueryWidget->slotCloseAll();
 
 	// Store opened call trees
@@ -1042,9 +1124,19 @@ bool KScope::slotCloseProject()
 	lstOpenFiles.setAutoDelete(true);
 	m_pProjMgr->setLastOpenFile(sLastOpenFile);
 	m_pProjMgr->setOpenFiles(lstOpenFiles);
+	m_pProjMgr->setBookmarks(lstBookmarks);
 	m_pProjMgr->setQueryFiles(slQueryFiles);
 	m_pProjMgr->setCallTreeFiles(slCallTreeFiles);
 	
+	// Destroy the make dialogue
+	if (m_pMakeDlg != NULL) {
+		m_pProjMgr->setMakeParams(m_pMakeDlg->getCommand(),
+			m_pMakeDlg->getDir());
+		
+		delete m_pMakeDlg;
+		m_pMakeDlg = NULL;
+	}
+	
 	// Close the project in the project manager, and terminate the Cscope
 	// process
 	m_pProjMgr->close();
@@ -1102,6 +1194,100 @@ void KScope::slotShowWelcome()
 	dlg.exec();
 }
 
+/**
+ * Handles the "Edit->Go To Tag" menu command.
+ * Sets the cursor to the edit box of the current tag list.
+ */
+void KScope::slotGotoTag()
+{
+	EditorPage* pPage;
+	
+	pPage = m_pEditTabs->getCurrentPage();
+	if (pPage)
+		pPage->setTagListFocus();
+}
+
+/**
+ * Reports the results of the Cscope verification script.
+ * This slot is connected to the done() signal emitted by the CscopeVerifier
+ * object constructed in verifyCscope().
+ */
+void KScope::slotCscopeVerified(bool bResult, uint nArgs)
+{
+	statusBar()->message(i18n("Verifying Cscope installation...Done"), 3000);
+	
+	// Mark the flag even if Cscope was not found, to avoid nagging the user
+	// (who may wish to use KScope even with Cscope disabled)
+	m_bCscopeVerified = true;
+
+	// Prompt the user in case Cscope is not properly installed
+	if (!bResult) {
+		KMessageBox::error(0, i18n("Cscope may not be properly installed on "
+			"this system.\nPlease check the Cscope path specified in KScope's "
+			"configuration dialogue."));
+		slotConfigure();
+		return;
+	}
+		
+	// Set the discoverred supported command-line arguments
+	CscopeFrontend::setSupArgs(nArgs);
+	
+	// Build the database, if required
+	if (m_bRebuildDB) {
+		m_bRebuildDB = false;
+		slotRebuildDB();
+	}
+}
+
+/**
+ * Handles the "Project->Make..." menu command.
+ * Displays the make dialogue.
+ */
+void KScope::slotProjectMake()
+{
+	QString sCmd, sDir;
+	
+	// Create the make dialogue, if it does not exist
+	if (m_pMakeDlg == NULL) {
+		// Create the dialogue
+		m_pMakeDlg = new MakeDlg();
+		
+		// Set make parameters for this project
+		m_pProjMgr->getMakeParams(sCmd, sDir);
+		m_pMakeDlg->setCommand(sCmd);
+		m_pMakeDlg->setDir(sDir);
+		
+		// Show the relevant source location when an error link is clicked
+		connect(m_pMakeDlg, SIGNAL(fileRequested(const QString&, uint)), this,
+			SLOT(slotShowEditor(const QString&, uint)));
+		
+		// Show the dialogue
+		m_pMakeDlg->show();
+	}
+	else if (m_pMakeDlg->isShown()) {
+		// The dialogue exists, and is visible, just raise it
+		m_pMakeDlg->raise();
+		m_pMakeDlg->setActiveWindow();
+	}
+	else {
+		// The dialogue exists but is closed, show it
+		m_pMakeDlg->show();
+	}
+}
+
+/**
+ * Handles the "Project->Remake..." menu command.
+ * Displays the make dialogue and runs the make command.
+ */
+void KScope::slotProjectRemake()
+{
+	// Make sure the make dialogue exists and is displayed
+	slotProjectMake();
+	
+	// Run the make command
+	m_pMakeDlg->slotMake();
+}
+
 /**
  * Prompts the user for a symbol to query.
  * Shows a dialog with a line edit widget, where the user can enter a symbol
@@ -1160,7 +1346,7 @@ EditorPage* KScope::addEditor(const QString& sFilePath)
 	// If the file name is given using a relative path, we need to convert
 	// it to an absolute one
 	if (sFilePath[0] != '/') {
-		sAbsFilePath = QDir::cleanDirPath(m_pProjMgr->getPath() + "/" +
+		sAbsFilePath = QDir::cleanDirPath(m_pProjMgr->getRoot() + "/" +
 			sFilePath);
 	}
 	else {
@@ -1217,6 +1403,19 @@ EditorPage* KScope::createEditorPage()
 	// Handle file drops
 	connect(pPage->getView(), SIGNAL(dropEventPass(QDropEvent*)), this,
 		SLOT(slotDropEvent(QDropEvent*)));
+		
+	// Manage bookmarks
+	connect(pPage, 
+		SIGNAL(bookmarkChanged(const QString&, uint, const QString&, bool)),
+		m_pQueryWidget,
+		SLOT(slotBookmarkChanged(const QString&, uint, const QString&, bool)));
+	
+	connect(pPage, SIGNAL(fileClosed(const QString&)), m_pQueryWidget,
+		SLOT(slotRemoveBookmarks(const QString&)));
+
+	// Apply per-project configuration
+	if (m_pProjMgr->getTabWidth() > 0)
+		pPage->setTabWidth(m_pProjMgr->getTabWidth());
 	
 	return pPage;
 }
@@ -1292,6 +1491,9 @@ void KScope::slotShowEditor(const QString& sFilePath, uint nLine)
 	pPage = addEditor(sFilePath);
 	if (pPage == NULL)
 		return;
+	
+	// Make sure the main window is visible
+	raise();
 
 	if (nLine != 0) {
 		// Set the cursor to the requested line
@@ -1417,11 +1619,28 @@ void KScope::slotBuildProgress(int nFiles, int nTotal)
 	}
 	
 	// Show progress information
-	sMsg = i18n("Building cross reference database...") + " " +
+	sMsg = i18n("Rebuilding the cross reference database...") + " " +
 		QString::number((nFiles * 100) / nTotal) + "%";
 	statusBar()->message(sMsg);
 }
 
+/**
+ * Reports to the user that Cscope has started building the inverted index.
+ * This slot is connected to the buildInvIndex() signal emitted by the 
+ * builder process.
+ */
+void KScope::slotBuildInvIndex()
+{
+	if (m_pProgressDlg) {
+		m_pProgressDlg->setLabel(i18n("Please wait while KScope builds the "
+			"inverted index"));
+		m_pProgressDlg->setIdle();
+		return;
+	}
+	
+	statusBar()->message(i18n("Rebuilding inverted index..."));
+}
+
 /**
  * Informs the user the database rebuild process has finished.
  * This slot is connected to the finished() signal emitted by the builder
@@ -1437,8 +1656,8 @@ void KScope::slotBuildFinished(uint)
 	}
 	
 	// Show a message in the status bar
-	statusBar()->message(i18n("Building cross reference database...Done!"),
-		3000);
+	statusBar()->message(i18n("Rebuilding the cross reference database..."
+		"Done!"), 3000);
 }
 
 /**
@@ -1463,8 +1682,8 @@ void KScope::slotBuildAborted()
 	}
 	
 	// Show a message in the status bar
-	statusBar()->message(i18n("Building cross reference database...Failed"),
-		3000);	
+	statusBar()->message(i18n("Rebuilding the cross reference database..."
+		"Failed"), 3000);	
 }
 
 /**
