@@ -48,6 +48,7 @@ class CscopeFrontend;
 class EditorPage;
 class ProgressDlg;
 class CscopeMsgDlg;
+class MakeDlg;
 class CallTreeManager;
 
 /**
@@ -75,9 +76,11 @@ public:
 	~KScope();
 
 	void openProject(const QString&);
+	void openLastProject();
 	bool openCscopeOut(const QString&);
 	void parseCmdLine(KCmdLineArgs *pArgs);
-		
+	void verifyCscope();
+	
 public slots:
 	void slotClose();
 
@@ -154,6 +157,21 @@ private:
 		loading/closing a number of editor parts. */
 	bool m_bUpdateGUI;
 	
+	/** Set to true after a shell script has verified that Cscope is installed
+		and correctly configured on this system.
+		No Cscope operations should be run if this flag is set to false. */
+	bool m_bCscopeVerified;
+	
+	/**
+	 * Used to postpone rebuilding of the database, until Cscope is ready.
+	 */
+	bool m_bRebuildDB;
+	
+	/**
+	 * A widget for running make.
+	 */
+	MakeDlg* m_pMakeDlg;
+	
 	void initActions();
 	void initMainWindow();
 	void initCscope();
@@ -162,6 +180,7 @@ private:
 	EditorPage* createEditorPage();
 	inline bool isAutoRebuildEnabled();
 	void loadOpenFiles();
+	void loadBookmarks();
 	void toggleQueryWindow(bool);
 	
 private slots:
@@ -193,7 +212,10 @@ private slots:
 	void slotExtEdit();
 	void slotCompleteSymbol();
 	void slotShowWelcome();
-	
+	void slotGotoTag();
+	void slotProjectMake();
+	void slotProjectRemake();
+
 	// Other slots
 	void slotProjectFilesChanged();
 	void slotFilesAdded(const QStringList&);
@@ -205,6 +227,7 @@ private slots:
 	void slotSetRebuildTimer(const QString&);
 	void slotCscopeError(const QString&);
 	void slotBuildProgress(int, int);
+	void slotBuildInvIndex();
 	void slotBuildFinished(uint);
 	void slotBuildAborted();
 	void slotApplyPref();
@@ -214,6 +237,7 @@ private slots:
 	void slotDropEvent(QDropEvent*);
 	void slotQueryDockClosed();
 	void slotFileViewDockClosed();
+	void slotCscopeVerified(bool, uint);
 };
 
 #endif
