@@ -51,11 +51,12 @@ public:
 
 	/** Identifiers for embedded pixmaps. */
 	enum PixName { SymFunc, SymVar, SymStruct, SymMacro, SymMember, SymEnum,
-		SymEnumerator, SymTypedef, SymLabel, SymUnknown, PIX_ARRAY_SIZE };
+		SymEnumerator, SymTypedef, SymLabel, SymInclude, SymUnknown,
+		PIX_ARRAY_SIZE };
 		
 	/** Identifiers for loadable pixmaps. */
-	enum LoadPixName { TabUnlocked, TabLocked, TabRW, TabRO, TabSave,
-		TabFileList, TabFileTree, TabList, ButtonSaveAs, ButtonZoomIn,
+	enum LoadPixName { TabUnlocked, TabLocked, TabBookmark, TabRW, TabRO, 
+		TabSave, TabFileList, TabFileTree, TabList, ButtonSaveAs, ButtonZoomIn,
 		ButtonZoomOut, ButtonRotate, ButtonPref, CalledTree,
 		CallingTree, CallGraph };
 	
