@@ -106,16 +106,21 @@ void NewProjectDlg::setProperties(const QString& sName, const QString& sPath,
 	m_pPathRequester->setURL(sPath);
 	m_pKernelCheck->setChecked(opt.bKernel);
 	m_pInvCheck->setChecked(opt.bInvIndex);
+	m_pNoCompCheck->setChecked(opt.bNoCompress);
+	m_pSlowPathCheck->setChecked(opt.bSlowPathDef);
 	
 	if (opt.nAutoRebuildTime >= 0) {
 		m_pAutoRebuildCheck->setChecked(true);
 		m_pAutoRebuildSpin->setValue(opt.nAutoRebuildTime);
-		slotAutoRebuildChanged(true);
 	}
 
 	if (opt.bACEnabled) {
 		m_pACCheck->setChecked(true);
-		slotAutoCompletionChanged(true);
+	}
+	
+	if (opt.nTabWidth > 0) {
+		m_pTabWidthCheck->setChecked(true);
+		m_pTabWidthSpin->setValue(opt.nTabWidth);
 	}
 	
 	// Initialise the auto-completion sub-dialogue
@@ -159,12 +164,19 @@ void NewProjectDlg::getOptions(ProjectManager::Options& opt)
 	opt.slFileTypes = m_slTypes;
 	opt.bKernel = m_pKernelCheck->isChecked();
 	opt.bInvIndex = m_pInvCheck->isChecked();
-	
+	opt.bNoCompress = m_pNoCompCheck->isChecked();
+	opt.bSlowPathDef = m_pSlowPathCheck->isChecked();
+		
 	if (m_pAutoRebuildCheck->isChecked())
 		opt.nAutoRebuildTime = m_pAutoRebuildSpin->value();
 	else
 		opt.nAutoRebuildTime = -1;
 		
+	if (m_pTabWidthCheck->isChecked())
+		opt.nTabWidth = m_pTabWidthSpin->value();
+	else
+		opt.nTabWidth = 0;
+		
 	opt.bACEnabled = m_pACCheck->isChecked();
 	opt.nACMinChars = m_pAutoCompDlg->m_nMinChars;
 	opt.nACDelay = m_pAutoCompDlg->m_nDelay;
@@ -267,31 +279,6 @@ void NewProjectDlg::slotAvailTypesChanged(const QString& sType)
 	m_pTypesEdit->setText(sType);
 }
 
-/**
- * Enables or disables the auto rebuild spin button based on the state of the
- * auto-rebuild check box.
- * This slot is connected to the toggled() signal of the auto rebuild
- * check-box.
- * @param	bOn	true if the check box is checked, false otherwise
- */
-void NewProjectDlg::slotAutoRebuildChanged(bool bOn)
-{
-	m_pAutoRebuildSpin->setEnabled(bOn);
-	m_pAutoRebuildLabel->setEnabled(bOn);
-}
-
-/**
- * Enables or disables the auto-completion button based on the state of the
- * auto-completion check box.
- * This slot is connected to the toggled() signal of the auto-completion
- * check-box.
- * @param	bOn	true if the check box is checked, false otherwise
- */
-void NewProjectDlg::slotAutoCompletionChanged(bool bOn)
-{
-	m_pACButton->setEnabled(bOn);
-}
-
 /**
  * Class constructor.
  * @param	pParent		The parent widget
