@@ -94,11 +94,9 @@ public:
 	
 protected slots:
 	virtual void accept();
-    virtual void slotAddType();
-    virtual void slotRemoveType();
-    virtual void slotAvailTypesChanged(const QString&);
-    virtual void slotAutoCompletionChanged(bool);
-    virtual void slotAutoRebuildChanged(bool);
+	virtual void slotAddType();
+	virtual void slotRemoveType();
+	virtual void slotAvailTypesChanged(const QString&);
 
 private:
 	/** The file MIME-types associated with the new project. */
