@@ -26,18 +26,27 @@
  ***************************************************************************/
 
 #include <unistd.h>
+#include <fcntl.h>
+#include <errno.h>
 #include <kmessagebox.h>
 #include <klocale.h>
 #include "projectmanager.h"
 #include "kscopeconfig.h"
+#include "cscopefrontend.h"
+#include "filesemaphore.h"
 
 #define CONFIG_VER 2
 
 #define DEF_IS_KERNEL		false
 #define DEF_INV_INDEX		true
+#define DEF_NO_COMPRESS		false
+#define DEF_SLOW_PATH		false
 #define DEF_AC_MIN_CHARS	3
 #define DEF_AC_DELAY		500
 #define DEF_AC_MAX_ENTRIES	100
+#define DEF_TAB_WIDTH		0 /* Use editor's default */
+
+#define PROJECT_MUTEX		"/kscope_proj_mutex"
 
 /**
  * Class constructor.
@@ -47,7 +56,10 @@ ProjectManager::ProjectManager() :
 	m_pConf(NULL),
 	m_bOpen(false),
 	m_bTempProj(false),
-	m_nAutoRebuildTime(-1)
+	m_nArgs(0),
+	m_nAutoRebuildTime(-1),
+	m_nTabWidth(0),
+	m_pSem(NULL)
 {
 }
 
@@ -135,6 +147,7 @@ bool ProjectManager::open(const QString& sPath)
 {
 	QString sConfFile;
 	Options opt;
+	FileSemaphore* pSem;
 	
 	// Associate the object with the project directory
 	m_dir.setPath(sPath);
@@ -161,21 +174,43 @@ bool ProjectManager::open(const QString& sPath)
 		KMessageBox::error(0, i18n("Cannot read project name"));
 		return false;
 	}
+
+	// Try to acquire a named semaphore for the project
+	pSem = new FileSemaphore(sPath + "/cscope.sem", PROJECT_MUTEX);
+	if (!pSem->acquire()) {
+		QString sMsg;
 		
+		sMsg = QString(i18n("Cannot acquire the project's semaphore\n"
+			"Error: %2\n").arg(errno));
+		KMessageBox::error(0, sMsg);
+		delete pSem;
+		return false;
+	}
+	
+	pSem->inc();
+	pSem->release();
+	
 	// Indicate that a project has been opened
 	m_bOpen = true;
 	m_bTempProj = false;
-
+	m_pSem = pSem;
+	
 	// Create the argument list for invoking Cscope
 	getOptions(opt);
 	if (opt.bKernel)
-		m_slArgs << "-k";
+		m_nArgs |= CscopeFrontend::Kernel;
 	if (opt.bInvIndex)
-		m_slArgs << "-q";
+		m_nArgs |= CscopeFrontend::InvIndex;
+	if (opt.bNoCompress)
+		m_nArgs |= CscopeFrontend::NoCompression;
+	if (opt.bSlowPathDef)
+		m_nArgs |= CscopeFrontend::SlowPathDef;
 	
 	// Get the auto-rebuild	time value
 	m_nAutoRebuildTime = opt.nAutoRebuildTime;
 	
+	m_nTabWidth = opt.nTabWidth;
+	
 	// Add to the list of recently opened projects
 	Config().addRecentProject(sPath);
 	
@@ -220,11 +255,24 @@ void ProjectManager::close()
 		m_pFiles = NULL;
 		delete m_pConf;
 		m_pConf = NULL;
+	
+		// Delete the project semaphore
+		if (m_pSem) {
+			// Decrement the reference counter
+			if (m_pSem->acquire()) {
+				m_pSem->dec();
+				m_pSem->release();
+			}
+			
+			delete m_pSem;
+			m_pSem = NULL;
+		}
 	}
 	
-	m_slArgs = QStringList();
+	m_nArgs = 0;
 	m_slSymHistory.clear();
 	m_nAutoRebuildTime = -1;
+	m_nTabWidth = 0;
 	m_bOpen = false;
 }
 
@@ -392,8 +440,11 @@ void ProjectManager::getOptions(Options& opt) const
 	opt.slFileTypes = m_pConf->readListEntry("FileTypes", ' ');
 	opt.bKernel = m_pConf->readBoolEntry("Kernel", DEF_IS_KERNEL);
 	opt.bInvIndex = m_pConf->readBoolEntry("InvIndex", DEF_INV_INDEX);
+	opt.bNoCompress = m_pConf->readBoolEntry("NoCompress", DEF_NO_COMPRESS);
+	opt.bSlowPathDef = m_pConf->readBoolEntry("SlowPathDef", DEF_SLOW_PATH);
 	opt.nAutoRebuildTime = m_pConf->readNumEntry("AutoRebuildTime");
-	
+	opt.nTabWidth = m_pConf->readUnsignedNumEntry("TabWidth");
+		
 	// Get auto-completion options
 	m_pConf->setGroup("AutoCompletion");
 	opt.bACEnabled = m_pConf->readBoolEntry("Enabled");
@@ -417,12 +468,20 @@ void ProjectManager::setOptions(const Options& opt)
 	// Write the options to the configuratio nfile
 	writeOptions(m_pConf, opt);	
 			
+	// Update project parameters
+	m_nAutoRebuildTime = opt.nAutoRebuildTime;
+	m_nTabWidth = opt.nTabWidth;
+	
 	// Create the argument list for invoking Cscope
-	m_slArgs = QStringList();
+	m_nArgs = 0;
 	if (opt.bKernel)
-		m_slArgs << "-k";
+		m_nArgs |= CscopeFrontend::Kernel;
 	if (opt.bInvIndex)
-		m_slArgs << "-q";
+		m_nArgs |= CscopeFrontend::InvIndex;
+	if (opt.bNoCompress)
+		m_nArgs |= CscopeFrontend::NoCompression;
+	if (opt.bSlowPathDef)
+		m_nArgs |= CscopeFrontend::SlowPathDef;
 }
 
 /**
@@ -432,7 +491,7 @@ void ProjectManager::setOptions(const Options& opt)
  * cursor inside each file (@see FileLocation).
  * @param	list	The list to fill
  */
-void ProjectManager::getOpenFiles(OpenFileList& list) const
+void ProjectManager::getOpenFiles(FileLocationList& list) const
 {
 	QStringList slEntry;
 	QStringList::Iterator itr;
@@ -464,7 +523,7 @@ void ProjectManager::getOpenFiles(OpenFileList& list) const
  * be read when the project is restored.
  * @param	list	The list of file paths to write
  */
-void ProjectManager::setOpenFiles(OpenFileList& list)
+void ProjectManager::setOpenFiles(FileLocationList& list)
 {
 	FileLocation* pLoc;
 	QString sLoc;
@@ -488,6 +547,64 @@ void ProjectManager::setOpenFiles(OpenFileList& list)
 	m_pConf->writeEntry("OpenFiles", slEntry);
 }
 
+/**
+ * Reads the stored session's bookmarks.
+ * The list is composed of the file paths and the current position of the
+ * cursor inside each file (@see FileLocation).
+ * @param	list	The list to fill
+ */
+void ProjectManager::getBookmarks(FileLocationList& list) const
+{
+	QStringList slEntry;
+	QStringList::Iterator itr;
+	QString sPath;
+	uint nLine, nCol;
+	
+	// Must have an open persistent project
+	if (!m_bOpen || m_bTempProj)
+		return;
+
+	// Read the file paths
+	m_pConf->setGroup("Session");
+	slEntry = m_pConf->readListEntry("Bookmarks");
+	
+	// Transform the entry into a list of file locations
+	for (itr = slEntry.begin(); itr != slEntry.end(); ++itr) {
+		sPath = (*itr).section(':', 0, 0);
+		nLine = (*itr).section(':', 1, 1).toUInt();
+		nCol = (*itr).section(':', 2, 2).toUInt();
+		list.append(new FileLocation(sPath, nLine, nCol));
+	}
+}
+
+/**
+ * Writes a list of bookmarks.
+ * @param	list	A list of file locations, each defining a bookmark
+ */
+void ProjectManager::setBookmarks(FileLocationList& list)
+{
+	FileLocation* pLoc;
+	QString sLoc;
+	QStringList slEntry;
+	
+	// Must have an open persistent project
+	if (!m_bOpen || m_bTempProj)
+		return;
+
+	// Turn the object list into a string list, so that it can be written in
+	// the configuration file
+	for (pLoc = list.first(); pLoc != NULL; pLoc = list.next()) {
+		sLoc = "";
+		QTextOStream(&sLoc) << pLoc->m_sPath << ":" << pLoc->m_nLine << ":" 
+			<< pLoc->m_nCol;
+		slEntry.append(sLoc);
+	}
+
+	// Write the file paths
+	m_pConf->setGroup("Session");
+	m_pConf->writeEntry("Bookmarks", slEntry);
+}
+
 /**
  * @return	The full path of the file that was active when the session was
  *			closed
@@ -636,6 +753,38 @@ void ProjectManager::setSymHistory(QStringList& slSymHistory)
 	m_slSymHistory = slSymHistory;
 }
 
+/**
+ * Retrieves parameters used to run a make-like command on this project.
+ * @param	sCmd	Holds the make command
+ * @param	sDir	Holds the directory in which to run the command
+ */
+void ProjectManager::getMakeParams(QString& sCmd, QString& sDir) const
+{
+	// Do nothing if there is no open project, or using a temporary project
+	if (!m_bOpen || m_bTempProj)
+		return;
+	
+	m_pConf->setGroup("Project");
+	sCmd = m_pConf->readEntry("MakeCommand", "make");
+	sDir = m_pConf->readEntry("MakeRoot", getRoot());
+}
+
+/**
+ * Sets parameters used to run a make-like command on this project.
+ * @param	sCmd	The make command
+ * @param	sDir	The directory in which to run the command
+ */
+void ProjectManager::setMakeParams(const QString& sCmd, const QString& sDir)
+{
+	// Do nothing if there is no open project, or using a temporary project
+	if (!m_bOpen || m_bTempProj)
+		return;
+	
+	m_pConf->setGroup("Project");
+	m_pConf->writeEntry("MakeCommand", sCmd);
+	m_pConf->writeEntry("MakeRoot", sDir);
+}
+
 /**
  * Determines if the cscope.out file for this project exists.
  * @return	true if the database exists, false otherwise
@@ -662,9 +811,12 @@ void ProjectManager::getDefOptions(Options& opt)
 	// Set other options
 	opt.bKernel = DEF_IS_KERNEL;
 	opt.bInvIndex = DEF_INV_INDEX;
+	opt.bNoCompress = DEF_NO_COMPRESS;
+	opt.bSlowPathDef = DEF_SLOW_PATH;
 	opt.nACMinChars = DEF_AC_MIN_CHARS;
 	opt.nACDelay = DEF_AC_DELAY;
 	opt.nACMaxEntries = DEF_AC_MAX_ENTRIES;
+	opt.nTabWidth = DEF_TAB_WIDTH;
 	
 	// Set profile-dependant options
 	if (Config().getSysProfile() == KScopeConfig::Fast) {
@@ -677,6 +829,26 @@ void ProjectManager::getDefOptions(Options& opt)
 	}
 }
 
+/**
+ * Determines whether a project is currently open by any running instance of 
+ * KScope.
+ * @param	sPath	The path of the project to check
+ * @return	true if the project is currently open, false otherwise
+ */
+bool ProjectManager::isGloballyOpen(const QString& sPath)
+{
+	FileSemaphore sem(sPath + "/cscope.sem", PROJECT_MUTEX);
+	bool bResult;
+	
+	if (!sem.acquire())
+		return false;
+		
+	bResult = (sem.getValue() > 0);
+	sem.release();
+	
+	return bResult;
+}
+
 /**
  * Creates the "cscope.files" and "cscope.proj" file objects for the given
  * project.
@@ -731,7 +903,10 @@ void ProjectManager::writeOptions(KConfig* pConf, const Options& opt)
 	pConf->writeEntry("FileTypes", opt.slFileTypes.join(" "));
 	pConf->writeEntry("Kernel", opt.bKernel);
 	pConf->writeEntry("InvIndex", opt.bInvIndex);		
+	pConf->writeEntry("NoCompress", opt.bNoCompress);		
+	pConf->writeEntry("SlowPathDef", opt.bSlowPathDef);		
 	pConf->writeEntry("AutoRebuildTime", opt.nAutoRebuildTime);
+	pConf->writeEntry("TabWidth", opt.nTabWidth);
 	
 	// Set auto-completion options
 	pConf->setGroup("AutoCompletion");
