@@ -132,7 +132,9 @@ struct FileLocation
 /**
  * A list of file locations used for restoring a session.
  */
-typedef QPtrList<FileLocation> OpenFileList;
+typedef QPtrList<FileLocation> FileLocationList;
+
+class FileSemaphore;
 
 /**
  * Maintains a KScope project.
@@ -171,6 +173,12 @@ public:
 		/** true if Cscope should build an inverted index. */
 		bool bInvIndex;
 		
+		/** true if the -c option for CScope should be used. */
+		bool bNoCompress;
+		
+		/** true if the -D option for CScope should be used. */
+		bool bSlowPathDef;
+		
 		/** The time, in milliseconds, after which the database should be
 			automatically rebuilt (-1 if this option is disabled). */
 		int nAutoRebuildTime;
@@ -187,6 +195,8 @@ public:
 		
 		/** Maximal number of entries for auto-completion. */
 		uint nACMaxEntries;
+		
+		uint nTabWidth;
 	};
 	
 	bool create(const QString&, const QString&, const Options&);
@@ -201,8 +211,10 @@ public:
 	QString getFileTypes() const;
 	void getOptions(Options&) const;
 	void setOptions(const Options&);
-	void getOpenFiles(OpenFileList&) const;
-	void setOpenFiles(OpenFileList&);
+	void getOpenFiles(FileLocationList&) const;
+	void setOpenFiles(FileLocationList&);
+	void getBookmarks(FileLocationList&) const;
+	void setBookmarks(FileLocationList&);
 	QString getLastOpenFile() const;
 	void setLastOpenFile(const QString&);
 	void getQueryFiles(QStringList&) const;
@@ -213,6 +225,8 @@ public:
 	void setRoot(const QString&);
 	void getSymHistory(QStringList&) const;
 	void setSymHistory(QStringList&);
+	void getMakeParams(QString&, QString&) const;
+	void setMakeParams(const QString&, const QString&);
 	bool dbExists();
 	
 	/**
@@ -240,10 +254,10 @@ public:
 	QString getPath() const { return m_dir.absPath(); }
 	
 	/**
-	 * @return	A list of command-line arguments to pass to a Cscope object,
-	 * based on the project's options
+	 * @return	Command-line arguments to pass to a Cscope object, based on
+	 * 			project's options
 	 */
-	const QStringList& getArgList() const { return m_slArgs; }
+	uint getArgs() const { return m_nArgs; }
 	
 	/**
 	 * @return	The time, in seconds, to wait before rebuilding the
@@ -251,8 +265,15 @@ public:
 	 */
 	int getAutoRebuildTime() const { return m_nAutoRebuildTime; }
 	
+	/**
+	 * @return	The tab width to use (0 to use the editor's default)
+	 */
+	uint getTabWidth() const { return m_nTabWidth; }
+	
 	static void getDefOptions(Options&);
 
+	static bool isGloballyOpen(const QString&);
+
 signals:
 	/**
 	 * Emitted when a set of files is added to the current project.
@@ -288,15 +309,22 @@ private:
 	
 	/** A list of Cscope command-line arguments based on the project's
 		options. */
-	QStringList m_slArgs;
+	uint m_nArgs;
 
 	/** The time, in seconds, to wait before rebuilding the cross-refernce
 	 	database. */
 	int m_nAutoRebuildTime;
 	
+	/** If non-zero, overrides the editor's configured tab width. */
+	uint m_nTabWidth;
+	
 	/** A list of symbols previously queried. */
 	QStringList m_slSymHistory;
 	
+	/** A named semaphore to synchronise multiple instances of the same 
+		project. */
+	FileSemaphore* m_pSem;
+	
 	static void initConfig(const QString&, QFile**, KConfig**);
 	static bool isCscopeOut(const QString&);
 	static void writeOptions(KConfig*, const Options&);
