@@ -44,6 +44,7 @@ QueryWidget::QueryWidget(QWidget* pParent, const char* szName) :
 	m_pLockAction(NULL),
 	m_pHistPage(NULL),
 	m_bHistEnabled(true),
+	m_pBMarkPage(NULL),
 	m_nQueryPages(0)
 {
 	// Pages can be closed by clicking their tabs
@@ -118,19 +119,19 @@ void QueryWidget::applyPrefs()
 }
 
 /**
- * Loads all queries that were locked when the project was closed.
+ * Loads all pages saved when the project was closed.
  * @param	sProjPath		The full path of the project directory
- * @param	slQueryFiles	The list of query file names to load
+ * @param	slFiles	The list of query file names to load
  */
-void QueryWidget::loadLockedQueries(const QString& sProjPath,
-	const QStringList& slQueryFiles)
+void QueryWidget::loadPages(const QString& sProjPath,
+	const QStringList& slFiles)
 {
 	QStringList::ConstIterator itr;
 	QueryPageBase* pPage;
 	QString sName;
 	
 	// Iterate through query files
-	for (itr = slQueryFiles.begin(); itr != slQueryFiles.end(); ++itr) {
+	for (itr = slFiles.begin(); itr != slFiles.end(); ++itr) {
 		// Set the target page, based on the file type (query or history)
 		if ((*itr).startsWith("History")) {
 			findHistoryPage();
@@ -150,12 +151,11 @@ void QueryWidget::loadLockedQueries(const QString& sProjPath,
 }
 
 /**
- * Saves all locked queries into files in the project directory.
+ * Stores all pages marked for saving into files in the project directory.
  * @param	sProjPath		The full path of the project directory
- * @param	slQueryFiles	Holds a list of query file names, upon return
+ * @param	slFiles			Holds a list of query file names, upon return
  */
-void QueryWidget::saveLockedQueries(const QString& sProjPath, 
-	QStringList& slQueryFiles)
+void QueryWidget::savePages(const QString& sProjPath, QStringList& slFiles)
 {
 	int nPageCount, i;
 	QueryPage* pPage;
@@ -165,13 +165,10 @@ void QueryWidget::saveLockedQueries(const QString& sProjPath,
 	nPageCount = m_pQueryTabs->count();
 	for (i = 0; i < nPageCount; i++) {
 		pPage = (QueryPage*)m_pQueryTabs->page(i);
-		if (pPage->isLocked()) {
+		if (pPage->shouldSave()) {
 			// Store this query page
-			sFileName = pPage->store(sProjPath);
-			
-			// A successful store operation returns a non-empty file name
-			if (!sFileName.isEmpty())
-				slQueryFiles.append(sFileName);
+			if (pPage->save(sProjPath, sFileName) && !sFileName.isEmpty())
+				slFiles.append(sFileName);
 		}
 	}
 }
@@ -353,26 +350,7 @@ void QueryWidget::slotCloseAll()
 	}
 	
 	m_pHistPage = NULL;
-}
-
-/**
- * Creates a new history page.
- */
-void QueryWidget::slotAddHistoryPage()
-{
-	QString sTitle;
-		
-	// Create the page
-	m_pHistPage = new HistoryPage(this);
-
-	// Add the page, and set it as the current one
-	m_pQueryTabs->insertTab(m_pHistPage, GET_PIXMAP(TabUnlocked), "");
-	setPageCaption(m_pHistPage);
-
-	// Emit the lineRequested() signal when a query record is selected on 
-	// this page
-	connect(m_pHistPage, SIGNAL(lineRequested(const QString&, uint)), this, 
-		SLOT(slotRequestLine(const QString&, uint)));
+	m_pBMarkPage = NULL;
 }
 
 /**
@@ -401,6 +379,53 @@ void QueryWidget::slotHistoryNext()
 	}
 }
 
+/**
+ * Called when a bookmark is added or removed on an editor page.
+ * Updates the active bookmarks page.
+ * This slot is connected to the bookmarkChanged() signal of each EditorPage
+ * object.
+ * @param	sPath	The file for which a bookmark was changed
+ * @param	nLine	The bookmark's line number
+ * @param	sText	The line's contents
+ * @param	bAdded	True if a bookmark was added, false if it was removed
+ */
+void QueryWidget::slotBookmarkChanged(const QString& sPath, uint nLine,
+	const QString& sText, bool bAdded)
+{
+	// Get the active bookmark page (add one if necessary)
+	findBookmarkPage();
+	
+	// Add or remove the bookmark, as required
+	if (bAdded)
+		m_pBMarkPage->addRecord(sPath, nLine, sText);
+	else
+		m_pBMarkPage->removeRecord(sPath, nLine);
+}
+
+/**
+ * Deletes all bookmarks for the given file.
+ * This slot is connected to the fileClosed() signal emitted by an EditorPage.
+ * @param	sPath	The path of the file for which bookmarks should be removed
+ */
+void QueryWidget::slotRemoveBookmarks(const QString& sPath)
+{
+	// Don't use findBookmarkPage(), since we don't want to show this page if
+	// it is currently hidden. All we care about is whether the page exists
+	if (m_pBMarkPage)
+		m_pBMarkPage->removeFile(sPath);
+}
+
+/**
+ * Handles the "Bookmarks->Browse Global Bookmarks" menu command.
+ * Sets the keyboard focus to the bookmarks page.
+ */
+void QueryWidget::slotBrowseBookmarks()
+{
+	findBookmarkPage();
+	setCurrentPage(m_pBMarkPage);
+	m_pBMarkPage->setFocus();
+}
+
 /**
  * Sets the active history page, if any, as the current page.
  */
@@ -423,13 +448,26 @@ void QueryWidget::setPageMenu(QPopupMenu* pMenu, KToggleAction* pAction)
 	m_pLockAction = pAction;
 }
 
+/**
+ * Fills a list with the locations of all defined bookmarks.
+ * @param	list	The list to fill
+ */
+void QueryWidget::getBookmarks(FileLocationList& list)
+{
+	if (!m_pBMarkPage)
+		return;
+		
+	m_pBMarkPage->fillList(list);
+}
+
 /**
  * Emits a signal indicating a certain source file and line number are
  * requested.
  * This slot is connected to the recordSelected() signal emitted by any of
  * the query pages. The signal emitted by this slot is used to display an
  * editor page at the requested line.
- * @param	pItem	The list item selected by the user
+ * @param	sFileName	The file's path
+ * @param	nLine		The requested line in the file
  */
 void QueryWidget::slotRequestLine(const QString& sFileName, uint nLine)
 {
@@ -464,6 +502,13 @@ void QueryWidget::slotCurrentChanged(QWidget* pWidget)
  */
 void QueryWidget::slotClosePage(QWidget* pPage)
 {
+	// The bookmarks page is only hidden, not deleted (so that bookmarks are
+	// not lost)
+	if (pPage == m_pBMarkPage) {
+		m_pQueryTabs->removePage(pPage);
+		return;
+	}
+	
 	// Prompt the user before closing a locked query
 	if (((QueryPage*)pPage)->isLocked()) {
 		if (KMessageBox::questionYesNo(NULL, i18n("You about about to close"
@@ -535,6 +580,9 @@ void QueryWidget::slotContextMenu(QWidget* pWidget, const QPoint& pt)
  */
 void QueryWidget::setPageLocked(QueryPageBase* pPage, bool bLock)
 {
+	if (!pPage->canLock())
+		return;
+		
 	// Set the locking state of the current page
 	pPage->setLocked(bLock);
 	m_pQueryTabs->setTabIconSet(pPage, bLock ? GET_PIXMAP(TabLocked) : 
@@ -597,6 +645,7 @@ void QueryWidget::findHistoryPage()
 {
 	HistoryPage* pPage;
 	int nPages, i;
+	QString sTitle;
 	
 	// First check if the active history page is unlocked
 	if (m_pHistPage != NULL && !m_pHistPage->isLocked())
@@ -613,7 +662,42 @@ void QueryWidget::findHistoryPage()
 	}
 
 	// Couldn't find an unlocked query page, create a new one
-	slotAddHistoryPage();
+	m_pHistPage = new HistoryPage(this);
+
+	// Add the page, and set it as the current one
+	m_pQueryTabs->insertTab(m_pHistPage, GET_PIXMAP(TabUnlocked), "");
+	setPageCaption(m_pHistPage);
+
+	// Emit the lineRequested() signal when a query record is selected on 
+	// this page
+	connect(m_pHistPage, SIGNAL(lineRequested(const QString&, uint)), this, 
+		SLOT(slotRequestLine(const QString&, uint)));
+}
+
+/**
+ * Ensures an active bookmarks page exists.
+ * There is at most one bookmarks page. If such a page does not already exist, 
+ * one is created.
+ */
+void QueryWidget::findBookmarkPage()
+{
+	QString sTitle;
+	
+	// Create a new page, if one does not already exist
+	if (!m_pBMarkPage) {
+		m_pBMarkPage = new BookmarkPage(this);
+		
+		// Emit the lineRequested() signal when a query record is selected on 
+		// this page
+		connect(m_pBMarkPage, SIGNAL(lineRequested(const QString&, uint)), this, 
+			SLOT(slotRequestLine(const QString&, uint)));
+	}
+
+	if (m_pQueryTabs->indexOf(m_pBMarkPage) == -1) {
+		// Add the page, and set it as the current one
+		m_pQueryTabs->insertTab(m_pBMarkPage, GET_PIXMAP(TabBookmark), "");
+		setPageCaption(m_pBMarkPage);
+	}
 }
 
 #include "querywidget.moc"
