@@ -35,6 +35,8 @@
 #include "tabwidget.h"
 #include "querypage.h"
 #include "historypage.h"
+#include "bookmarkpage.h"
+#include "projectmanager.h"
 
 /**
  * A tabbed-window holding Cscope query results pages.
@@ -52,12 +54,18 @@ public:
 	void addQueryPage();
 	void initQuery(uint, const QString&);
 	void applyPrefs();
-	void loadLockedQueries(const QString&, const QStringList&);
-	void saveLockedQueries(const QString&, QStringList&);
+	void loadPages(const QString&, const QStringList&);
+	void savePages(const QString&, QStringList&);
 	void addHistoryRecord(const QString&, uint, const QString&);
 	void selectActiveHistory();
 	void setPageMenu(QPopupMenu*, KToggleAction*);
+	void getBookmarks(FileLocationList&);
 	
+	/**
+	 * Enables/disables new history items.
+	 * @param	bEnabled	true to enable new history items, false to 
+	 *						disable
+	 */
 	void setHistoryEnabled(bool bEnabled) { m_bHistEnabled = bEnabled; }
 
 public slots:
@@ -69,9 +77,11 @@ public slots:
 	void slotPrevResult();
 	void slotCloseCurrent();
 	void slotCloseAll();
-	void slotAddHistoryPage();
 	void slotHistoryPrev();
 	void slotHistoryNext();
+	void slotBookmarkChanged(const QString&, uint, const QString&, bool);
+	void slotRemoveBookmarks(const QString&);
+	void slotBrowseBookmarks();
 	
 signals:
 	/**
@@ -92,6 +102,7 @@ private:
 		query, etc.). */
 	QPopupMenu* m_pPageMenu;
 	
+	/** A toggle-like action for changing the locked state of a query. */
 	KToggleAction* m_pLockAction;
 	
 	/** The active history page. */
@@ -101,6 +112,9 @@ private:
 		history page. */
 	bool m_bHistEnabled;
 	
+	/** The bookmarks page. */
+	BookmarkPage* m_pBMarkPage;
+	
 	/** The number of query pages currently open. */
 	int m_nQueryPages;
 	
@@ -132,7 +146,8 @@ private:
 	
 	void setPageLocked(QueryPageBase*, bool);
 	void findQueryPage();
-	void findHistoryPage();	
+	void findHistoryPage();
+	void findBookmarkPage();
 		
 private slots:
 	void slotRequestLine(const QString&, uint);
